from __future__ import print_function

import argparse

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from irco import models, utils, countries
from irco.parsers import scopus, compendex, wos
from irco.logging import get_logger, sentry


def get_records(source, pipeline):
    for path in utils.get_file_list(source):
        sentry.extra_context({
            'input_file': path,
        })
        with pipeline.open(path) as fh:
            for record in pipeline.process(fh):
                yield record
        del sentry.context['extra']['input_file']


def import_records(engine, records):
    Session = sessionmaker(bind=engine)
    imported = 0
    ignored = 0
    for record in records:
        session = Session()

        publication = (session.query(models.Publication)
                       .filter_by(unique_source_id=record.unique_source_id)
                       .first())

        if publication:
            session.close()
            ignored += 1
            continue

        publication = models.Publication(
            year=record['year'],
            title=record['title'],
            type=record.get('type', None),
            times_cited=record.get('times_cited', None),
            unique_source_id=record.unique_source_id,
            unparsed_record_format=record.format,
            unparsed_record_value=record.raw_value,
        )
        session.add(publication)

        institutions = {}

        for k, v in enumerate(record['institutions']):
            try:
                country = countries.get_institution_country(v)
            except countries.CountryNotFound:
                country = None

            instance, _ = models.get_or_create(
                session, models.Institution, defaults={'country': country},
                name=v
            )
            institutions[k] = (instance, v)

        for i, (name, affiliation) in enumerate(record['authors']):
            author = models.Person(name=name)

            if affiliation is not None:
                institution, raw = institutions[affiliation]
            else:
                institution, raw = None, None

            affiliated_author = models.AffiliatedAuthor(
                order=1,
                is_corresponding=(
                    record.get('corresponding_author', None) == i),
                unparsed_institution_name=raw,
                institution=institution,
                unparsed_person_name=name,
                author=author,
                publication=publication,
            )
            session.add(affiliated_author)

        session.commit()
        session.close()
        imported += 1

    return imported, ignored


def main():
    log = get_logger()

    pipelines = {
        'compendex': compendex.pipeline,
        'scopus': scopus.pipeline,
        'wos': wos.pipeline,
    }

    argparser = argparse.ArgumentParser('irco-import')
    argparser.add_argument('-v', '--verbose', action='store_true')
    argparser.add_argument('-i', '--input-format', choices=pipelines)
    argparser.add_argument('-e', '--encoding', default='utf8')
    argparser.add_argument('source', nargs='+')
    argparser.add_argument('database')

    args = argparser.parse_args()

    sentry.context.merge({
        'tags': {
            'command': 'irco-import',
            'input_format': args.input_format,
        },
        'extra': {
            'parsed_arguments': args.__dict__,
        }
    })

    log.info('arguments_parsed', args=args)

    pipeline = pipelines[args.input_format](encoding=args.encoding)
    engine = create_engine(args.database, echo=args.verbose)
    Session = sessionmaker(bind=engine)

    count_before = Session().query(models.Publication).count()
    records = get_records(args.source, pipeline)
    imported, ignored = import_records(engine, records)
    count_after = Session().query(models.Publication).count()

    pipeline.add_metric('imported_records', 'Records added to the database',
                        imported)
    pipeline.add_metric('ignored_records',
                        'Ignored records (already imported)', ignored)
    pipeline.add_metric('before_import', 'Records count before import',
                        count_before)
    pipeline.add_metric('after_import', 'Records count after import',
                        count_after)
    print()
    print(pipeline.report())



"""
Configuration management for the different module components.
"""


import os
try:
    import configparser
except ImportError:
    import ConfigParser as configparser


MODULE = __name__.split('.', 1)[0]

DEFAULT_FILES = [
    os.path.join(os.path.dirname(__file__), 'default-settings.ini'),
    '/etc/{0}/{0}.ini'.format(MODULE),
    os.path.expanduser('~/.{}.ini'.format(MODULE)),
    os.path.join(os.path.realpath('.'), '{}.ini'.format(MODULE)),
]


def load_config(path=None, defaults=None):
    """
    Loads and parses an INI style configuration file using Python's built-in
    ConfigParser module.

    If path is specified, load it.

    If ``defaults`` (a list of strings) is given, try to load each entry as a
    file, without throwing any error if the operation fails.

    If ``defaults`` is not given, the following locations listed in the
    DEFAULT_FILES constant are tried.

    To completely disable defaults loading, pass in an empty list or ``False``.

    Returns the SafeConfigParser instance used to load and parse the files.
    """

    if defaults is None:
        defaults = DEFAULT_FILES

    config = configparser.SafeConfigParser(allow_no_value=True)

    if defaults:
        config.read(defaults)

    if path:
        with open(path) as fh:
            config.readfp(fh)

    return config


settings = load_config()



from __future__ import absolute_import, print_function
import logging
import os
import sys


logging.basicConfig(
    filename=os.environ.get('IRCO_LOGFILE', 'irco.log'),
    level=logging.INFO,
)


from structlog import get_logger, configure
from structlog.stdlib import LoggerFactory


__all__ = ['get_logger']


configure(logger_factory=LoggerFactory())


from raven import Client
from irco.conf import settings
from irco import __version__


def make_excepthook(client):
    def excepthook(*exc_info):
        ident = client.get_ident(client.captureException(exc_info))
        r = sys.__excepthook__(*exc_info)
        print('-' * 80)
        print('  This exception was logged remotely. Please use the following'
              ' ID when\n  seeking support:', ident)
        print('-' * 80)
        return r
    return excepthook

enabled = settings.getboolean('logging', 'sentry')
dsn = settings.get('logging', 'sentry_dsn')

if enabled:
    sentry = Client(dsn)
    sentry.tags_context({
        'version': __version__,
    })
    if dsn:
        sys.excepthook = make_excepthook(sentry)
else:
    # Hide disabled sentry reporting
    logging.getLogger('raven.base.Client').setLevel(logging.WARNING)
    sentry = Client('')



import argparse
import collections
import re

import pycountry

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from irco import models, utils
from irco.logging import get_logger


def prep_name(name):
    name = re.sub(r'[.\'-]+', '', name.lower())
    name = re.sub(r'[^a-zA-Z0-9]+', ' ', name.lower())
    return re.split(r'\s+', name)


def main():
    log = get_logger()

    argparser = argparse.ArgumentParser('irco-geolocate')
    argparser.add_argument('-v', '--verbose', action='store_true')
    argparser.add_argument('database')
    argparser.add_argument('institution', nargs='*', type=int)

    args = argparser.parse_args()

    log.info('arguments_parsed', args=args)

    engine = create_engine(args.database, echo=args.verbose)
    Session = sessionmaker(bind=engine)
    session = Session()

    words = collections.Counter()
    country_words = set()
    words_to_country = {}

    for country in pycountry.countries.objects:
        ws = set(country.name.lower().split())
        for w in ws:
            if len(w) > 3:
                words_to_country.setdefault(w, []).append(country)
        country_words |= ws

    for institution, in session.query(models.Institution.name):
        words.update(w for w in prep_name(institution) if len(w) > 3)

    for w in country_words:
        try:
            del words[w]
        except KeyError:
            pass

    common = set()
    for w, freq in words.most_common(20):
        print '{:5}: {}'.format(freq, w)
        common.add(w)

    print

    import nltk
    import pprint
    import requests
    for institution_id in args.institution:
        institution = session.query(models.Institution).get(institution_id)
        print institution.name
        print

        name = set(prep_name(institution.name))
        name -= common
        countries = collections.Counter()
        for word in name:
            if word in words_to_country:
                for c in words_to_country[word]:
                    countries[c] += 1. / len(c.name.split())
                #country_candidates.extend(words_to_country[word])
        for w, freq in countries.most_common(10):
            print '{:5}: {}'.format(freq, w.name)

        print

        payload = {
            'query': institution.name,
            'sensor': 'false',
            'key': 'AIzaSyCLG_ILPIr6mauKBDk5nNiLhCJ9VHb2t4E',
        }
        r1 = requests.get('https://maps.googleapis.com/maps/api/place/textsearch/json', params=payload)

        j = r1.json()

        if j['status'] == 'ZERO_RESULTS':
            print 'Not found'
            continue

        print j

        payload = {
            'reference': r1.json()['results'][0]['reference'],
            'sensor': 'false',
            'key': 'AIzaSyCLG_ILPIr6mauKBDk5nNiLhCJ9VHb2t4E',
        }
        r2 = requests.get('https://maps.googleapis.com/maps/api/place/details/json', params=payload)

        res = r2.json()['result']
        print res['name']
        pprint.pprint(res['address_components'])


        #tokens = nltk.word_tokenize(institution.name)
        #print tokens
        #tagged = nltk.pos_tag(tokens)
        #print tagged
        #entities = nltk.chunk.ne_chunk(tagged)
        #print entities



import tablib

from collections import OrderedDict


class Table(object):

    def __init__(self, notset=None):
        self.headers = OrderedDict()
        self.data = []
        self.notset = notset

    def get_header_id(self, k):
        try:
            return self.headers[k]
        except KeyError:
            header_id = self.headers[k] = len(self.headers)
            return header_id

    def add(self, data):
        row = {}
        for k, v in data.iteritems():
            row[self.get_header_id(k)] = v
        self.data.append(row)

    def rows(self):
        for row in self.data:
            l = []
            for index in self.headers.itervalues():
                l.append(row.get(index, self.notset))
            yield tuple(l)

    def dataset(self):
        dataset = tablib.Dataset(headers=self.headers.keys())
        for row in self.rows():
            dataset.append(row)
        return dataset



import abc
import inspect
import collections
import codecs


class IgnoreRecord(Exception):
    pass


class Tokenizer(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def tokenize(self, stream):
        pass


class Parser(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def parse_record(self, record):
        pass


class Processor(object):
    __metaclass__ = abc.ABCMeta

    def initialize(self, pipeline):
        self.pipeline = pipeline

    @abc.abstractmethod
    def process_record(self, record):
        pass


class ValueProcessor(Processor):

    @abc.abstractproperty
    def key(self):
        pass

    @abc.abstractmethod
    def process_value(self, value):
        pass

    def process_record(self, record):
        if isinstance(self.key, basestring):
            oldkey = newkey = self.key
        else:
            oldkey, newkey = self.key
        record[newkey] = self.process_value(record[oldkey])

        if newkey != oldkey:
            del record[oldkey]

        return record


class Pipeline(object):
    def __init__(self, tokenizer, parser, processors, encoding='utf8'):
        self.tokenizer = tokenizer
        self.parser = parser
        self.processors = processors
        self.encoding = encoding
        self._opened_files = 0
        self._records = 0
        self._processed = 0
        self._metrics = collections.OrderedDict()

        for p in processors:
            p.initialize(self)

    def _format_error(self, record, e, processor=None):
        frm = inspect.trace()[-1]
        mod = inspect.getmodule(frm[0])
        line = inspect.getsourcelines(frm[0])[1] + 1
        title = record.get('title', '')
        if len(title) > 50:
            title = title[:50] + '...'
        if processor:
            print '=== ERROR WHILE PROCESSING RECORD ' + '=' * 45
            print ' Processor class: {}'.format(processor.__class__.__name__)
        else:
            print '=== ERROR WHILE PARSING RECORD ===' + '=' * 45
        print '          Module: {}:{}'.format(mod.__name__, line)
        print '       Exception: {}'.format(type(e).__name__)
        print '         Message: {}'.format(e)
        print '     Source file: {0[0]}:{0[1]}'.format(record.source)
        print '    Record title: {}'.format(title)
        print '-' * 80

    def _apply_processor(self, p, records):
        return (self._process_record(p, r) for r in records if r is not None)

    def _process_record(self, processor, record):
        try:
            return processor.process_record(record)
        except Exception as e:
            self._format_error(record, e, processor=processor)

    def _parse_record(self, record):
        self._records += 1
        try:
            return self.parser.parse_record(record)
        except Exception as e:
            self._format_error(record, e)

    def add_metric(self, key, name, val=0):
        self._metrics[key] = [name, val]

    def inc_metric(self, key):
        self._metrics[key][1] += 1

    def open(self, path):
        self._opened_files += 1
        return codecs.open(path, 'rb', encoding=self.encoding)

    def report(self):
        fields = [
            ('Processed files', self._opened_files),
            ('Total records found', self._records),
            ('Records ignored due to errors', self._records - self._processed),
            ('Successfully processed records', self._processed),
        ] + self._metrics.values()

        l = max(len(f[0]) for f in fields)
        s = ''
        s += '-' * 80 + '\n'
        for k, v in fields:
            s += '{:>{}s}: {}\n'.format(k, l+1, v)
        s += '-' * 80
        return s

    def process(self, stream):
        records = self.tokenizer.tokenize(stream)
        records = (self._parse_record(r) for r in records)
        for p in self.processors:
            records = self._apply_processor(p, records)
        for r in records:
            if r is not None:
                self._processed += 1
                yield r


class Record(dict):
    def __init__(self, value_format, raw_value):
        self.format = value_format
        self.raw_value = raw_value



#!/usr/bin/env python

import os
import re
from setuptools import setup, find_packages


class Setup(object):
    @staticmethod
    def read(fname, fail_silently=False):
        """
        Utility function to read the content of the given file.
        """
        try:
            return open(os.path.join(os.path.dirname(__file__), fname)).read()
        except:
            if not fail_silently:
                raise
            return ''

    @staticmethod
    def requirements(fname):
        """
        Utility function to create a list of requirements from the output of
        the pip freeze command saved in a text file.
        """
        packages = Setup.read(fname, fail_silently=True).split('\n')
        packages = (p.strip() for p in packages)
        packages = (p for p in packages if p and not p.startswith('#'))
        return list(packages)

    @staticmethod
    def get_files(*bases):
        """
        Utility function to list all files in a data directory.
        """
        for base in bases:
            basedir, _ = base.split('.', 1)
            base = os.path.join(os.path.dirname(__file__), *base.split('.'))

            rem = len(os.path.dirname(base)) + len(basedir) + 2

            for root, dirs, files in os.walk(base):
                for name in files:
                    yield os.path.join(basedir, root, name)[rem:]

    @staticmethod
    def version():
        data = Setup.read(os.path.join('irco', '__init__.py'))
        version = (re.search(u"__version__\s*=\s*u?'([^']+)'", data)
                   .group(1).strip())
        return version

    @staticmethod
    def test_links():
        # Test if hardlinks work. This is a workaround until
        # http://bugs.python.org/issue8876 is solved
        if hasattr(os, 'link'):
            tempfile = __file__ + '.tmp'
            try:
                os.link(__file__, tempfile)
            except OSError as e:
                if e.errno == 1:  # Operation not permitted
                    del os.link
                else:
                    raise
            finally:
                if os.path.exists(tempfile):
                    os.remove(tempfile)


Setup.test_links()

setup(name='irco',
      version=Setup.version(),
      description='International Research Collaboration Graphs',
      author='Jonathan Stoppani',
      author_email='jonathan@stoppani.name',
      url='https://github.com/GaretJax/irco',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=Setup.requirements('requirements.txt'),
      entry_points=Setup.read('entry-points.ini', True),
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ])





class RecordException(Exception):
    def __init__(self, record):
        self.record = record


class NoCorrespondingAuthor(RecordException):
    pass


class NoAffiliationField(RecordException):
    pass


class NoAffiliationMatch(RecordException):
    pass



import os
import glob
import importlib


def get_graphs():
    graphs = []
    for path in glob.glob(os.path.join(os.path.dirname(__file__), '*.py')):
        name = os.path.basename(path[:-3])
        if not name.startswith('_'):
            graphs.append(name)
    return tuple(graphs)


def get_graph(name):
    return importlib.import_module('irco.graphs.{}'.format(name))
